let () =
  let open Command_line in
  begin
    set_header_doc (Sys.argv.(0) ^ " - QCaml command");
    set_description_doc "Runs a Hartree-Fock calculation";
    set_specs
      [ { short='b' ; long="basis" ; opt=Mandatory;
          arg=With_arg "<string>";
          doc="Name of the file containing the basis set"; } ;

        { short='x' ; long="xyz" ; opt=Mandatory;
          arg=With_arg "<string>";
          doc="Name of the file containing the nuclear coordinates in xyz format"; } ;

        { short='m' ; long="multiplicity" ; opt=Optional;
          arg=With_arg "<int>";
          doc="Spin multiplicity (2S+1). Default is singlet"; } ;

        { short='c' ; long="charge" ; opt=Optional;
          arg=With_arg "<int>";
          doc="Total charge of the molecule. Default is 0"; } ;
      ]
  end;

  (* Handle options *)
  let basis_file  = 
    match Command_line.get "basis" with
    | Some x -> x
    | None -> assert false
  in

  let nuclei_file = 
    match Command_line.get "xyz" with
    | Some x -> x
    | None -> assert false
  in

  let charge = 
    match Command_line.get "charge" with
    | Some x -> int_of_string x
    | None   -> 0
  in

  let multiplicity =
    match Command_line.get "multiplicity" with
    | Some x -> int_of_string x
    | None -> 1
  in

  let s =
    Simulation.of_filenames ~charge ~multiplicity ~nuclei:nuclei_file basis_file
  in

  HartreeFock.make s 
  |> HartreeFock.to_string
  |> print_endline


